package me.desht.pneumaticcraft.client.render;

import com.mojang.blaze3d.matrix.MatrixStack;
import com.mojang.blaze3d.systems.RenderSystem;
import me.desht.pneumaticcraft.client.util.ProgressingLine;
import me.desht.pneumaticcraft.client.util.RenderUtils;
import me.desht.pneumaticcraft.common.hacking.secstation.HackSimulation;
import org.lwjgl.opengl.GL11;

import static me.desht.pneumaticcraft.common.hacking.secstation.HackSimulation.GRID_WIDTH;

public class RenderHackSimulation {
    private final int baseX;
    private final int baseY;
    private final int nodeSpacing;

    public RenderHackSimulation(int baseX, int baseY, int nodeSpacing) {
        this.baseX = baseX;
        this.baseY = baseY;
        this.nodeSpacing = nodeSpacing;
    }

    public void render(MatrixStack matrixStack, HackSimulation hackSimulation, int color) {
        RenderSystem.enableBlend();
        RenderSystem.disableTexture();
        RenderSystem.blendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE_MINUS_SRC_ALPHA);
        for (HackSimulation.ConnectionEntry entry : hackSimulation.allConnections) {
            RenderUtils.renderProgressingLineGUI(matrixStack, makeProgressingLine(entry), color, 3F);
        }
        RenderSystem.enableTexture();
        RenderSystem.disableBlend();
    }

    private ProgressingLine makeProgressingLine(HackSimulation.ConnectionEntry entry) {
        int xOff = 0;
        int yOff = 0;
        int delta = entry.from - entry.to;
        if (delta == GRID_WIDTH) {
            xOff = 1;
        } else if (delta == -GRID_WIDTH) {
            xOff = -1;
        } else {
            yOff = delta < 0 ? -1 : 1;
        }
        int startX = baseX + xOff + entry.from % GRID_WIDTH * nodeSpacing;
        int startY = baseY + yOff + entry.from / GRID_WIDTH * nodeSpacing;
        int endX = baseX + xOff + entry.to % GRID_WIDTH * nodeSpacing;
        int endY = baseY + yOff + entry.to / GRID_WIDTH * nodeSpacing;
        return new ProgressingLine(startX, startY, endX, endY).setProgress(entry.progress);
    }
}
